@@ -1,21 +1,30 @@
+from ..custom_logging import log
+from typing import Optional, List, Dict, Any
+
+import asyncio
+import json
+import uuid
+from ..utils.mime import guess_mime_type
+import traceback
+
+_DISCOVERYENGINE_AVAILABLE = False
 try:
     from google.api_core.client_options import ClientOptions
     from google.cloud import discoveryengine
     from google.api_core.retry import Retry, if_exception_type
     from google.api_core.exceptions import ResourceExhausted, AlreadyExists
-    from google.cloud.discoveryengine_v1alpha import SearchResponse, Chunk
+    from google.api_core.exceptions import GoogleAPIError
+    from google.cloud.discoveryengine_v1 import SearchResponse, Chunk
+    from google.cloud.discoveryengine_v1.services.search_service.pagers import SearchPager, SearchAsyncPager
+    _DISCOVERYENGINE_AVAILABLE = True
 except ImportError:
     ClientOptions = None
     discoveryengine = None
     Chunk = None
     SearchResponse = None
+    SearchAsyncPager = None
+    SearchPager = None
 
-from ..custom_logging import log
-from typing import Optional, List
-import asyncio
-import json
-import uuid
-from ..utils.mime import guess_mime_type
 
 class DiscoveryEngineClient:
     """
@@ -80,8 +89,10 @@ class DiscoveryEngineClient:
             self.async_search_client = discoveryengine.SearchServiceAsyncClient(client_options=client_options)
         except RuntimeError:
             # No event loop in non-async environment, set async client to None
-            log.info("No event loop detected; skipping async client initialization")
+            log.info("No event loop detected; skipping Discoveryengine async client initialization")
             self.async_search_client = None
+        
+        log.info(f"Discovery Engine client initialized with {self.project_id=}, {self.data_store_id=}, {self.location=}")
 
     @classmethod
     def my_retry(cls):
@@ -344,12 +355,15 @@ class DiscoveryEngineClient:
         return search_response
     
     def chunk_format(self, chunk):
+
         return (
                     f"# {chunk.id}\n"
                     f"{chunk.content}\n"
-                    f"## metadata\n"
+                    f"## {chunk.id} metadata\n"
+                    f"Relevance score: {chunk.relevance_score}\n"
                     f"Document URI: {chunk.document_metadata.uri}\n"
                     f"Document Title: {chunk.document_metadata.title}\n"
+                    f"Document Metadata: {dict(chunk.document_metadata.struct_data)}\n"
                 )        
 
     def process_chunks(self, response):
@@ -357,7 +371,8 @@ class DiscoveryEngineClient:
 
         # Check if the response contains results
         if not hasattr(response, 'results') or not response.results:
-            raise ValueError(f'No results found in response: {response=}')
+            log.info(f'No results found in response: {response=}')
+            return []
         
         # Iterate through each result in the response
         for result in response.results:
@@ -386,7 +401,8 @@ class DiscoveryEngineClient:
 
         # Check if the response contains results
         if not hasattr(response, 'results') or not response.results:
-            raise ValueError(f'No results found in response: {response=}')
+            log.info(f'No results found in response: {response=}')
+            return []
         
         # Iterate through each result in the response
         for result in response.results:
@@ -485,6 +501,7 @@ class DiscoveryEngineClient:
             return doc_client.import_documents(request=request)
         
         try:
+            log.debug(f"Requesting import of documents: {request=}")
             operation = import_documents_with_retry(self.doc_client, request)
         except ResourceExhausted as e:
             log.error(f"DiscoveryEngine Operation failed after retries due to quota exceeded: {e}")
@@ -605,6 +622,7 @@ class DiscoveryEngineClient:
             reconciliation_mode=discoveryengine.ImportDocumentsRequest.ReconciliationMode.INCREMENTAL,
         )
 
+        log.debug(f"Making import_document_request: {request}")
         return self._import_document_request(request)
 
     def _create_unique_gsuri_docid(self, gcs_uri:str):
@@ -627,6 +645,7 @@ class DiscoveryEngineClient:
             str: The operation name.
         """
         try:
+            log.info(f"Importing doc with metadata: {gcs_uri=}, {metadata=}")
             # 1. Generate a unique document ID
             document_id = self._create_unique_gsuri_docid(gcs_uri)
 
@@ -657,7 +676,7 @@ class DiscoveryEngineClient:
             return self._import_document_request(request)
 
         except Exception as e:
-            log.error(f"Error importing document with metadata: {e}")
+            log.error(f"Error importing document with metadata: {e} {traceback.format_exc()}")
             raise e
     
     def get_mime_type(self, uri:str):
@@ -726,6 +745,67 @@ class DiscoveryEngineClient:
         log.info("Discovery engine response object")
         return search_response
 
+    async def async_search_with_filters(self, query, filter_str=None,
+                            num_previous_chunks=3, num_next_chunks=3, 
+                            page_size=10, parse_chunks_to_string=True, 
+                            serving_config="default_serving_config",
+                            data_store_ids: Optional[List[str]] = None):
+        """
+        Searches with a generic filter string asynchronously.
+
+        Args:
+            query (str): The search query.
+            filter_str (str, optional): The filter string to apply (e.g., "source LIKE 'my_source' AND eventTime > TIMESTAMP('2024-01-01')").
+            #... other parameters from get_chunks
+
+        Returns:
+            discoveryengine.SearchResponse or str: The search response object or string of chunks.
+        """
+
+        serving_config_path = self.async_search_client.serving_config_path(
+            self.project_id,
+            self.location,
+            self.data_store_id,
+            serving_config
+        )
+
+        search_request = discoveryengine.SearchRequest(
+            serving_config=serving_config_path,
+            query=query,
+            page_size=page_size, 
+            content_search_spec=discoveryengine.SearchRequest.ContentSearchSpec(
+                search_result_mode="CHUNKS", 
+                chunk_spec=discoveryengine.SearchRequest.ContentSearchSpec.ChunkSpec(
+                    num_previous_chunks=num_previous_chunks,
+                    num_next_chunks=num_next_chunks,
+                ),
+            ),
+            filter=filter_str # name:'ANY("king kong")'
+        )
+
+        if data_store_ids:
+            search_request.data_store_specs = [
+                discoveryengine.SearchRequest.DataStoreSpec(
+                    data_store=self._search_data_store_path(data_store_id, serving_config=serving_config)
+                )
+                for data_store_id in data_store_ids
+            ]
+
+        log.info(f"Discovery engine request with filter: {search_request=}")
+        try:
+            search_response = await self.async_search_client.search(search_request)
+        except Exception as e:
+            log.info(f"No results {search_request.data_store_specs=}: {str(e)}")
+            return None
+        
+        if parse_chunks_to_string:
+            big_string = await self.async_process_chunks(search_response)
+            log.info(f"Discovery engine chunks string sample: {big_string[:100]}")
+            return big_string
+        
+        log.info("Discovery engine response object")
+        return search_response
+
     def search_by_objectId_and_or_date(self, query, objectId=None, date=None, **kwargs):
         """
         Searches and filters by objectId (exact match) and/or date.
@@ -751,3 +831,323 @@ class DiscoveryEngineClient:
         else:
             # No filters, perform regular search
             return self.search_with_filters(query, **kwargs)
+
+    async def async_search_by_objectId_and_or_date(self, query, objectId=None, date=None, **kwargs):
+        """
+        Searches and filters by objectId (exact match) and/or date asynchronously.
+
+        Args:
+            query (str): The search query.
+            objectId (str, optional): The exact objectId to filter by.
+            date (str, optional): The literal_iso_8601_datetime_format date to filter by e.g. 2025-02-24T12:25:30.123Z
+            **kwargs: Additional keyword arguments to pass to `async_search_with_filters`.
+
+        Returns:
+            list: A list of search results.
+        """
+        filter_clauses = []
+        if objectId:
+            filter_clauses.append(f'objectId: ANY("{objectId}")')
+        if date:
+            filter_clauses.append(f'eventTime >= "{date}"')
+
+        if filter_clauses:
+            filter_str = " AND ".join(filter_clauses)  # Combine with AND
+            return await self.async_search_with_filters(query, filter_str, **kwargs)
+        else:
+            # No filters, perform regular search
+            return await self.async_search_with_filters(query, **kwargs)
+
+    # --- NEW ENGINE SEARCH METHODS ---
+    def search_engine(
+        self,
+        search_query: str,
+        engine_id: str,
+        serving_config_id: str = "default_config",
+        page_size: int = 10,
+        return_snippet: bool = True,
+        summary_result_count: int = 5,
+        include_citations: bool = True,
+        custom_prompt: Optional[str] = None,
+        model_version: str = "stable",
+        query_expansion_level: "discoveryengine.SearchRequest.QueryExpansionSpec.Condition" # type: ignore
+             # Default value assignment MUST be conditional
+             = discoveryengine.SearchRequest.QueryExpansionSpec.Condition.AUTO if _DISCOVERYENGINE_AVAILABLE else None,
+
+        spell_correction_mode: "discoveryengine.SearchRequest.SpellCorrectionSpec.Mode" # type: ignore
+             # Default value assignment MUST be conditional
+             = discoveryengine.SearchRequest.SpellCorrectionSpec.Mode.AUTO if _DISCOVERYENGINE_AVAILABLE else None,
+        filter_str: Optional[str] = None,
+        boost_spec: Optional["discoveryengine.SearchRequest.BoostSpec"] = None, # Okay if default is None # type: ignore
+        params: Optional[Dict[str, Any]] = None,
+        user_pseudo_id: Optional[str] = None,
+        collection_id: str = "default_collection" # Needed for engine path
+    ):
+        """
+        Performs a search against a specified Discovery Engine Search Engine.
+
+        Allows configuration for snippets, summaries, query expansion, spell correction, etc.
+
+        Args:
+            search_query: The user's search query string.
+            engine_id: The ID of the search engine to query.
+            serving_config_id: The ID of the specific serving config for the engine.
+            page_size: Maximum number of results per page.
+            return_snippet: Whether to request snippets in the results.
+            summary_result_count: Number of results to use for generating a summary.
+                                  Set to 0 to disable summaries.
+            include_citations: Whether summaries should include citations.
+            custom_prompt: A custom preamble text to guide the summary generation model.
+            model_version: The version of the summary generation model (e.g., "stable").
+            query_expansion_level: Level of query expansion to apply (AUTO, DISABLED).
+            spell_correction_mode: Mode for spell correction (AUTO, SUGGEST).
+            filter_str: An optional filter string to apply to the search.
+            boost_spec: Optional boost specification object.
+            params: Optional dictionary of custom parameters.
+            user_pseudo_id: Optional unique identifier for the user/session.
+            custom_fine_tuning_spec: Optional spec to use a fine-tuned model.
+            collection_id: The collection ID associated with the engine.
+
+        Returns:
+            A SearchPager object to iterate through results, or None if an error occurs.
+        
+        Example:
+        client = DiscoveryEngineClient(
+            project_id=PROJECT_ID,
+            data_store_id=DATA_STORE_ID,
+            location=LOCATION
+        )
+
+        # --- Example: Searching an Engine ---
+        search_query_engine = "tell me about search engines"
+        log.info(f"\n--- Searching Engine: {ENGINE_ID} ---")
+        engine_pager = client.search_engine(
+            search_query=search_query_engine,
+            engine_id=ENGINE_ID,
+            summary_result_count=3 # Request a summary for 3 results
+        )
+
+        if engine_pager:
+            results_found = False
+            # Iterate through pages to get summary/results
+            for page in engine_pager.pages:
+                 results_found = True
+                 if page.summary:
+                      print(f"\nSearch Summary:\n{page.summary.summary_text}\n")
+                      # Citations are part of the summary object if requested
+                      if page.summary.summary_with_metadata:
+                           print("Summary Metadata/Citations:")
+                           for citation in page.summary.summary_with_metadata.citations:
+                                print(f"  - Citation Source: {citation.sources}")
+                           # Access references etc. if needed
+
+                 print("Results on this page:")
+                 for result in page.results:
+                      print(f"  ID: {result.document.id}")
+                      print(f"  Name: {result.document.name}")
+                      # Access snippet if available in result.document.derived_struct_data['snippets']
+                      # Access other document fields as needed (struct_data, etc.)
+                      # print(f"  Raw Result: {result}") # For detailed inspection
+                 print("-" * 10)
+
+            if not results_found:
+                 print("No results found for the engine search.")
+        else:
+            print(f"Engine search failed for query: '{search_query_engine}'")
+
+        """
+
+        if not _DISCOVERYENGINE_AVAILABLE:
+            log.error("Discovery Engine library not available at runtime.")
+            return None
+        
+        try:
+            # Construct the serving config path for an ENGINE
+            # Note: The client library path helper is for data stores/serving configs within them.
+            # We need the path for an engine's serving config.
+            serving_config_path = (
+                f"projects/{self.project_id}/locations/{self.location}/"
+                f"collections/{collection_id}/engines/{engine_id}/"
+                f"servingConfigs/{serving_config_id}"
+            )
+            log.info(f"Using Engine Serving Config Path: {serving_config_path}")
+
+            # --- Build ContentSearchSpec ---
+            snippet_spec = None
+            if return_snippet:
+                snippet_spec = discoveryengine.SearchRequest.ContentSearchSpec.SnippetSpec(
+                    return_snippet=True
+                )
+
+            summary_spec = None
+            if summary_result_count > 0:
+                model_prompt_spec = None
+                if custom_prompt:
+                    model_prompt_spec = discoveryengine.SearchRequest.ContentSearchSpec.SummarySpec.ModelPromptSpec(
+                        preamble=custom_prompt
+                    )
+
+                summary_spec = discoveryengine.SearchRequest.ContentSearchSpec.SummarySpec(
+                    summary_result_count=summary_result_count,
+                    include_citations=include_citations,
+                    ignore_adversarial_query=True,  # Default from original sample
+                    ignore_non_summary_seeking_query=True, # Default from original sample
+                    model_prompt_spec=model_prompt_spec,
+                    model_spec=discoveryengine.SearchRequest.ContentSearchSpec.SummarySpec.ModelSpec(
+                        version=model_version
+                    ),
+                )
+
+            content_search_spec = discoveryengine.SearchRequest.ContentSearchSpec(
+                snippet_spec=snippet_spec,
+                summary_spec=summary_spec,
+                # Unlike get_chunks, don't specify search_result_mode or chunk_spec here
+                # unless specifically needed for a document/snippet search use case.
+            )
+
+            # --- Build other Specs ---
+            query_expansion_spec = discoveryengine.SearchRequest.QueryExpansionSpec(
+                condition=query_expansion_level
+            )
+
+            spell_correction_spec = discoveryengine.SearchRequest.SpellCorrectionSpec(
+                mode=spell_correction_mode
+            )
+
+            # --- Build SearchRequest ---
+            request = discoveryengine.SearchRequest(
+                serving_config=serving_config_path,
+                query=search_query,
+                page_size=page_size,
+                content_search_spec=content_search_spec,
+                query_expansion_spec=query_expansion_spec,
+                spell_correction_spec=spell_correction_spec,
+                filter=filter_str,
+                boost_spec=boost_spec,
+                params=params,
+                user_pseudo_id=user_pseudo_id,
+                # Add other relevant fields like facet_specs if needed
+            )
+
+            log.info(f"Searching engine '{engine_id}' with request: {request}")
+            response_pager = self.search_client.search(request)
+            log.info(f"Search successful for query '{search_query}' against engine '{engine_id}'.")
+            return response_pager
+
+        except GoogleAPIError as e:
+            log.error(f"API error searching engine '{engine_id}': {e}")
+            return None
+        except Exception as e:
+            log.error(f"Unexpected error searching engine '{engine_id}': {e}\n{traceback.format_exc()}")
+            return None
+
+    async def async_search_engine(
+        self,
+        search_query: str,
+        engine_id: str,
+        serving_config_id: str = "default_config",
+        page_size: int = 10,
+        return_snippet: bool = True,
+        summary_result_count: int = 5,
+        include_citations: bool = True,
+        custom_prompt: Optional[str] = None,
+        model_version: str = "stable",
+        query_expansion_level: "discoveryengine.SearchRequest.QueryExpansionSpec.Condition" # type: ignore
+             # Default value assignment MUST be conditional
+             = discoveryengine.SearchRequest.QueryExpansionSpec.Condition.AUTO if _DISCOVERYENGINE_AVAILABLE else None,
+
+        spell_correction_mode: "discoveryengine.SearchRequest.SpellCorrectionSpec.Mode" # type: ignore
+             # Default value assignment MUST be conditional
+             = discoveryengine.SearchRequest.SpellCorrectionSpec.Mode.AUTO if _DISCOVERYENGINE_AVAILABLE else None,
+        filter_str: Optional[str] = None,
+        boost_spec: Optional["discoveryengine.SearchRequest.BoostSpec"] = None, # Okay if default is None # type: ignore
+        params: Optional[Dict[str, Any]] = None,
+        user_pseudo_id: Optional[str] = None,
+        collection_id: str = "default_collection"
+    ):
+        """
+        Performs an asynchronous search against a specified Discovery Engine Search Engine.
+
+        Allows configuration for snippets, summaries, query expansion, spell correction, etc.
+
+        Args:
+            (Same arguments as the synchronous search_engine method)
+
+        Returns:
+            An SearchAsyncPager object to iterate through results asynchronously,
+            or None if an error occurs or the async client is not available.
+        """
+        if not self.async_search_client:
+             log.error("Cannot call async_search_engine: Async client not initialized.")
+             raise RuntimeError("Async client not initialized. Ensure class is instantiated within an async context.")
+
+        try:
+            # Construct the serving config path for an ENGINE (same as sync)
+            serving_config_path = (
+                f"projects/{self.project_id}/locations/{self.location}/"
+                f"collections/{collection_id}/engines/{engine_id}/"
+                f"servingConfigs/{serving_config_id}"
+            )
+            log.info(f"Using Async Engine Serving Config Path: {serving_config_path}")
+
+            # --- Build Specs (same logic as sync version) ---
+            snippet_spec = None
+            if return_snippet:
+                snippet_spec = discoveryengine.SearchRequest.ContentSearchSpec.SnippetSpec(...)
+            summary_spec = None
+            if summary_result_count > 0:
+                model_prompt_spec = None
+                if custom_prompt:
+                    model_prompt_spec = discoveryengine.SearchRequest.ContentSearchSpec.SummarySpec.ModelPromptSpec(
+                        preamble=custom_prompt
+                    )
+                summary_spec = discoveryengine.SearchRequest.ContentSearchSpec.SummarySpec(
+                    summary_result_count=summary_result_count,
+                    include_citations=include_citations,
+                    ignore_adversarial_query=True,  # Default from original sample
+                    ignore_non_summary_seeking_query=True, # Default from original sample
+                    model_prompt_spec=model_prompt_spec,
+                    model_spec=discoveryengine.SearchRequest.ContentSearchSpec.SummarySpec.ModelSpec(
+                        version=model_version
+                    ),
+                )
+
+            content_search_spec = discoveryengine.SearchRequest.ContentSearchSpec(
+                snippet_spec=snippet_spec,
+                summary_spec=summary_spec,
+            )
+            query_expansion_spec = discoveryengine.SearchRequest.QueryExpansionSpec(
+                condition=query_expansion_level
+            )
+
+            spell_correction_spec = discoveryengine.SearchRequest.SpellCorrectionSpec(
+                mode=spell_correction_mode
+            )
+
+            # --- Build SearchRequest (same logic as sync version) ---
+            request = discoveryengine.SearchRequest(
+                serving_config=serving_config_path,
+                query=search_query,
+                page_size=page_size,
+                content_search_spec=content_search_spec,
+                query_expansion_spec=query_expansion_spec,
+                spell_correction_spec=spell_correction_spec,
+                filter=filter_str,
+                boost_spec=boost_spec,
+                params=params,
+                user_pseudo_id=user_pseudo_id,
+            )
+
+            log.info(f"Async searching engine '{engine_id}' with request: {request}")
+            response_pager = await self.async_search_client.search(request)
+            log.info(f"Async search successful for query '{search_query}' against engine '{engine_id}'.")
+            return response_pager
+
+        except GoogleAPIError as e:
+            log.error(f"Async API error searching engine '{engine_id}': {e}")
+            return None
+        except Exception as e:
+            log.error(f"Async unexpected error searching engine '{engine_id}': {e}\n{traceback.format_exc()}")
+            return None
+
+# --- End of DiscoveryEngineClient class ---
\ No newline at end of file
